#!/bin/bash

usage() {
    echo "Usage:  $0 [-o output.ps] [subdir-list]"
    echo ""
    echo "This command plots the several aspects of the performance of the"
    echo "MPI test routine.  Gnuplot is required for this script to "
    echo "function properly."
    echo ""
    echo "If the subdirectory list is not given, it will be probed for."
}


psopt="?"

opt=0
j=0
while true; do
    getopts "ho:" opt;

    case "$opt" in
        "?")
            # go on
            break;
        ;;
        "o")
            psfile="$OPTARG"
            psopt=1
            let j=$j+2;
        ;;
        "h")
            usage;
            exit;
        ;;
    esac;
done
shift $j


plotfile=`mktemp $0.XXXXXX`

echo "plotting $plotfile"


getlastarg() {
    local num
    let num=$#-1
    shift $num
    last=$1
}

giveplot() {
    printf "set title '$2'\n\n" >> $plotfile

    ptype=(`seq 1 1 100`)
    #ptype=(1 5 2 6 3 7)
    j=0

    echo "plot \\" >> $plotfile
    for i in $args; do
        $1 "$i/node_info" ${ptype[$j]}

        if [ "$i" != "$last" ]; then
            echo ", \\" >> $plotfile
        else
            echo "" >> $plotfile
        fi

        let j++
    done;

    if [ "$psopt" == "?" ]; then
        printf "pause -1 'Press enter to continue';\n\n" >> $plotfile
    fi
}


if [ "$#" -ge 1 ]; then
    getlastarg $*
    args="$*"
else
    # we'll try to automatically test for the subdirs to use
    TRY=`ls |grep "\<[0-9]*\>"`
    last=""
    for i in $TRY; do
        if [ -d "$i" -a -f "$i/node_info" ]; then
            args="$args $i"
            last="$i"
        fi;
    done;

    if [ "$last" = "" ]; then
        usage;
        exit;
    fi;
fi;

echo "#!/usr/bin/gnuplot" >> $plotfile
echo "set mouse" >> $plotfile
echo "set pointsize 2" >> $plotfile
if [ "$psopt" != "?" ]; then
    echo "set output '$psfile'" >> $plotfile
    echo "set term post enhanced color \"Times-Roman\" 15" >> $plotfile
fi


bd() {
    echo "    '$1' using 4 w lp pt $2 t '$1 min', \\" >> $plotfile
    printf "    '$1' using 1 w lp pt $2 t '$1 max'" >> $plotfile
}
giveplot "bd"  "max/min boundaries"


actbd() {
    echo "    '$1' using 10 w lp pt $2 t '$1 min', \\" >> $plotfile
    printf "    '$1' using 7 w lp pt $2 t '$1 max'" >> $plotfile
}
giveplot "actbd"  "actual max/min boundaries"


cputime() {
    printf "    '$1' using 13 w lp t '$1'" >> $plotfile
}
giveplot "cputime" "cpu_time"


numpart() {
    printf "    '$1' using 16 w lp t '$1'" >> $plotfile
}
giveplot "numpart" "number particles"


bdshift() {
    echo "    '$1' using (-\$14) w lp pt $2 t '$1 left', \\" >> $plotfile
    printf "    '$1' using 15 w lp pt $2 t '$1 right'" >> $plotfile
}
giveplot "bdshift" "boundary shift"


transit() {
    printf "    '$1' using 17 w lp t '$1'" >> $plotfile
}
giveplot "transit" "max_transit"


tot_shift_err() {
    if [ "$1" = "0/node_info" ]; then
        printf "      '$1' using 18 w lp t '$1 : total shift error', \\\\\n" >> $plotfile
        printf "      '$1' using 19 w lp t '$1 : stddev(total shift error)'" >> $plotfile
    else
        printf "      0 t ''" >> $plotfile
    fi;
}
giveplot "tot_shift_err" "total shift err (only active root info is valid)"


echo "set output" >> $plotfile

gnuplot $plotfile

rm -f $plotfile
