// -*- c++ -*-
// $Id: boundeddata.C,v 1.2 2005/06/07 10:00:33 olsonse Exp $
/*
 * Copyright 2004-2005 Spencer Olson
 *
 * $Log: boundeddata.C,v $
 * Revision 1.2  2005/06/07 10:00:33  olsonse
 * Many changes over the course of time
 *
 * Revision 1.1.1.1  2005/01/08 04:57:26  olsonse
 * Initial CVS import (finally).
 *
 *
 */


#include <dsmc/DSMCData.h>
#include <dsmc/OctreeInterpolator.h>
#include <dsmc/OctreeMonitor.h>
#include "interp.h"
#include <fstream>
#include <octave/oct.h>
#include <octave/pager.h>

class DataReaper {
  public:
    inline DataReaper() : dtP(NULL), val(NULL) {}
    ColumnVector * dtP;
    Matrix * val;

    /** startMeasure is prescribed for use in OctreeMonitor. */
    template <class T>
    inline void startMeasure(const T & o, const uint8_t & ptype) {
    }

    /** Collect the data. */
    inline void addMeasure(const DSMCData & d) {
        ColumnVector & dt = *dtP;
        RowVector row(3 + dt.length());
        row(0) = d.pos[X];
        row(1) = d.pos[Y];
        row(2) = d.pos[Z];

        for (int i = 0; i < dt.length(); i++) {
            row(3+i) = d.data.indx[(int)dt(i)];
        }/* for */

#if 0
        octave_stdout << "old matrix: " << val->rows() << " X " << val->columns() << std::endl;
        octave_stdout << "new length: " << row.length() << std::endl;
#endif

        /* this should append the data row to the end of the matrix. */
        val->resize(1+val->rows(),row.length());
        val->insert(row,val->rows()-1,0);
    }

    /** do nothing but satisfy OctreeMonitor. */
    template <class T>
    inline void finishMeasure(const T & o) {
    }
};

DEFUN_DLD (boundeddata, args, ,
"  Matrix = boundeddata(iid, indx, col, bounds)\n"
"  Find datanode values for data column bounded by specified bounds.\n"
"\n"
"Copyright 2004 Spencer Olson:\n"
"\n"
"  Who knows what to believe with this code yet. :-)\n"
"\n"
"  iid    : id of interpolators as returned by loadointerp\n"
"  indx   : interpolator index\n"
"  col    : data column(s) to interpolate (this can be a vector)\n"
"  bounds : matrix of boundary specifications such as:\n"
"           [ [xmin,xmax]; [ymin,ymax]; [zmin,zmax] ]\n"
"\n"
"  Matrix : Returned value is a matrix of all bounded values\n"
"           including position information such as:\n"
"           [[ x, y, z, data]; nextrow; nextrow;...]\n"
) {
    if (args.length() != 4) {
        print_usage("boundeddata");
        return octave_value();
    }

    /* now create interpolator for each block of data */
    interpolator_vector * interpolators = (interpolator_vector *)((long)(args(0).double_value ()));

    if (!interpolators || !interpolators->testValid()) {
        octave_stdout << "invalid iid" << std::endl;
        print_usage("boundeddata");
        return octave_value();
    }

    unsigned int indx (args(1).int_value());
    if (indx >= interpolators->i.size()) {
        octave_stdout << "invalid indx" << std::endl;
        print_usage("boundeddata");
        return octave_value();
    }

    ColumnVector dt (args(2).vector_value());
    Matrix bounds (args(3).matrix_value());

    if (dt.length() < 1) {
        octave_stdout << "you must want SOME data column" << std::endl;
        print_usage("boundeddata");
        return octave_value();
    }

    if ( bounds.columns() != 2 || bounds.rows() != 3) {
        octave_stdout << "boundary informatin must be a 3 X 2 matrix" << std::endl;
        print_usage("boundeddata");
        return octave_value();
    }

    Matrix val;

    octree::OctreeMonitor<DataReaper,DSMCData> monitor;
    monitor.data.dtP = & dt;
    monitor.data.val = &val;

    monitor.vol.min[X] = bounds(0,0);
    monitor.vol.min[Y] = bounds(1,0);
    monitor.vol.min[Z] = bounds(2,0);
    monitor.vol.max[X] = bounds(0,1);
    monitor.vol.max[Y] = bounds(1,1);
    monitor.vol.max[Z] = bounds(2,1);
#if 0
    octave_stdout << " [xb=[" << monitor.vol.min[X] << ',' << monitor.vol.max[X] << ']'
                  << " [yb=[" << monitor.vol.min[Y] << ',' << monitor.vol.max[Y] << "];"
                  << " [zb=[" << monitor.vol.min[Z] << ',' << monitor.vol.max[Z] << "];"
                  << std::endl;
#endif

    monitor.monitor(*(interpolators->i[indx]->getOctree()), (const DSMCData**)interpolators->i[indx]->getParticles());

    return octave_value (val);
}

