// -*- c++ -*-
// $Id: nearestdata.C,v 1.1.1.1 2005/01/08 04:57:26 olsonse Exp $
/*
 * Copyright 2004-2005 Spencer Olson
 *
 * $Log: nearestdata.C,v $
 * Revision 1.1.1.1  2005/01/08 04:57:26  olsonse
 * Initial CVS import (finally).
 *
 *
 */


#include <fstream>
#include <dsmc/DSMCData.h>
#include <dsmc/OctreeInterpolator.h>
#include "interp.h"
#include <octave/oct.h>
#include <octave/pager.h>

DEFUN_DLD (nearestdata, args, ,
"  Matrix = nearestdata(iid, indx, col, x, y, z)\n"
"  Find nearest datanode value for data column at specified coordinates.\n"
"\n"
"Copyright 2004 Spencer Olson:\n"
"\n"
"  Who knows what to believe with this code yet. :-)\n"
"\n"
"  iid : id of interpolators as returned by loadointerp\n"
"  indx: interpolator index\n"
"  col : data column to interpolate\n"
"  x   : x position to interpolate to\n"
"  y   : y position to interpolate to\n"
"  z   : z position to interpolate to\n"
"  x, y, and z must have the same size\n"
) {
    if (args.length() != 6) {
        print_usage("nearestdata");
        return octave_value();
    }

    /* now create interpolator for each block of data */
    interpolator_vector * interpolators = (interpolator_vector *)((long)(args(0).double_value ()));

    if (!interpolators || !interpolators->testValid()) {
        octave_stdout << "invalid iid" << std::endl;
        print_usage("nearestdata");
        return octave_value();
    }

    unsigned int indx (args(1).int_value());
    if (indx >= interpolators->i.size()) {
        octave_stdout << "invalid indx" << std::endl;
        print_usage("nearestdata");
        return octave_value();
    }

    int dt (args(2).int_value());
    Matrix x (args(3).matrix_value()),
           y (args(4).matrix_value()),
           z (args(5).matrix_value());

    if ( x.columns() != y.columns() || x.columns() != z.columns() ||
         x.rows() != y.rows() || x.rows() != z.rows()) {
        octave_stdout << "x, y, and z must be the same size" << std::endl;
        print_usage("nearestdata");
        return octave_value();
    }

    Matrix val(x.rows(),x.columns());

    for (int i = 0; i < x.columns(); i++) {
        for (int j = 0; j < x.rows(); j++) {
            val(j,i) =
                interpolators->i[indx]->getParticles()[
                    interpolators->i[indx]->getOctree()->
                        findNearestLeaf(x(j,i), y(j,i), z(j,i)).low
                ]->data.indx[dt];
        }/* for */
    }/* for */

    return octave_value (val);
}
