// -*- c++ -*-
// $Id: FieldData.C,v 1.2 2005/06/07 10:00:07 olsonse Exp $
/*
 * Copyright 2004 Spencer Olson
 *
 * $Log: FieldData.C,v $
 * Revision 1.2  2005/06/07 10:00:07  olsonse
 * Many changes over the course of time
 *
 * Revision 1.1.1.1  2005/01/08 04:57:25  olsonse
 * Initial CVS import (finally).
 *
 */

#include "FieldData.h"
#include <olson-tools/physical.h>
#include <vector>
#include <iterator>

FieldData::FieldData() {
    zeroFields();
}

FieldData::FieldData(const FieldData & d) {
    memcpy(pos,d.pos, sizeof(pos));
    memcpy(&data, &d.data, sizeof (data));
    radiusOfInfluence = d.radiusOfInfluence;
    nearestNeighbors = d.nearestNeighbors;

    /* still set this to NULL */
    octreeNode = NULL;
    next = NULL;
}

FieldData::FieldData(const octree::Octree<FieldData> & o, FieldData * a[]) {
    zeroFields();
    /* set the position of this data point to the center of mass of the
     * particles in the octree node.  Not that it really matters.  This is
     * really only for debugging purposes.
     */
    o.copyRcm(pos);
}

/* TODO:  Pick some good value for the mem-limit */
REDEF_NEW_DEL_C(FieldData,/* freed mem limit (instances) = */ 100000);

std::ostream & operator<<(std::ostream & output,const FieldData & q){
    output << q.pos[X] << '\t' << q.pos[Y] << '\t' << q.pos[Z] << '\t'
           << q.data.name.field[X] << '\t'
           << q.data.name.field[Y] << '\t'
           << q.data.name.field[Z] << '\t'
           << q.data.name.magnitude << '\t'
           << q.data.name.gradient;
    return output;
} // operator<< FieldData

inline bool read_past_comment(std::istream & input) {
    char testchar = input.peek();
    /* read past spaces and endlines. */
    while (testchar == ' ' || testchar == '\n') {
        (void)input.get();
        testchar = input.peek();
    }

    if (testchar == '#') {
        /* read past comments. */
	std::stringbuf testbuf;
	input.get(testbuf, '\n');
        return true;
    }

    return false;
}

std::istream & operator>>(std::istream & input, FieldValue & v) {
    while (read_past_comment(input));
    return input >> v.val[0]
                 >> v.val[1]
                 >> v.val[2]
                 >> v.val[3]
                 >> v.val[4]
                 >> v.val[5]
                 >> v.val[6]
                 >> v.val[7];
}

std::ostream & operator<<(std::ostream & output, const FieldValue & v) {
    return output << v.val[0] << ' '
                  << v.val[1] << ' '
                  << v.val[2] << ' '
                  << v.val[3] << ' '
                  << v.val[4] << ' '
                  << v.val[5] << ' '
                  << v.val[6] << ' '
                  << v.val[7];
}

FieldData * readFieldData(std::istream & input, std::vector<FieldValue> * viP) {
    std::vector<FieldValue> * vP = NULL;
    if (viP) { vP = viP; }
    else { vP = new std::vector<FieldValue>; }

    std::vector<FieldValue> & V = *vP;

    FieldData * fieldData = NULL,
              **fdP = &fieldData;

    copy(std::istream_iterator<FieldValue>(input),
         std::istream_iterator<FieldValue>(),
         std::back_inserter(V));   

    for (std::vector<FieldValue>::iterator i = V.begin(); i < V.end();
         i++, fdP = &((*fdP)->next)) {
        FieldValue & f = *i;
        (*fdP) = new FieldData;
        (*fdP)->pos[X]               = f.val[0];
        (*fdP)->pos[Y]               = f.val[1];
        (*fdP)->pos[Z]               = f.val[2];
        (*fdP)->data.name.field[X]        = f.val[3];
        (*fdP)->data.name.field[Y]        = f.val[4];
        (*fdP)->data.name.field[Z]        = f.val[5];
        (*fdP)->data.name.magnitude       = f.val[6];
        (*fdP)->data.name.gradient        = f.val[7];
    }

#if 0
    if (V.size() < 101) {
        cout << "And the input was:  " << endl;
        copy(V.begin(), V.end(), ostream_iterator<FieldValue>(cout, "\n"));
    }
#endif

    if (!input.eof()) {
        std::cerr << "Error reading data point number "
        << V.size()
        << std::endl;
    }

    std::cout << "Read in " << V.size() << " data points" << std::endl;

    if (!viP) delete vP;
    return fieldData;
}

