// -*- c++ -*-
// $Id: OctreeContainer.C,v 1.2 2005/06/07 10:00:10 olsonse Exp $
/*
 * Copyright 2004 Spencer Olson
 *
 * $Log: OctreeContainer.C,v $
 * Revision 1.2  2005/06/07 10:00:10  olsonse
 * Many changes over the course of time
 *
 * Revision 1.1.1.1  2005/01/08 04:57:25  olsonse
 * Initial CVS import (finally).
 *
 *
 */

#include "OctreeContainer.h"

namespace octree {

/* TODO:  Pick some good value for the mem-limit */

REDEF_NEW_DEL_C(RemovalQueueNode,/* freed mem limit (instances) = */ 100000);

OctreeTemplate
int OctreeContainer<OctreeTemplateArgs>::particleSpaceIncrement = 500;

OctreeTemplate
OctreeContainer<OctreeTemplateArgs>::OctreeContainer(enum STYLE o):
                               particles(NULL),
			       nParticles(0),
			       octree(NULL),
			       particleSpaceAllocated(0),
                               container_style(o),
                               removal_queue(NULL) { };

OctreeTemplate
OctreeContainer<OctreeTemplateArgs>::~OctreeContainer() {
    flushRemovalQueue();
    freeParticles();
    freeOctree();
}

OctreeTemplate
int OctreeContainer<OctreeTemplateArgs>::addParticles(_Particle * input) THROWS(std::bad_alloc) {
    int total_processed = 0;
    _Particle * iPtr = input;
    while (input) {
	total_processed++;
	iPtr = input->next;

	/* check to see if we have space enough to store this particle */
	if (nParticles >= particleSpaceAllocated) {
	    /* allocate some more space */
	    particleSpaceAllocated += mytype::particleSpaceIncrement;
	    particles = (_Particle **)realloc(particles, particleSpaceAllocated * sizeof(void*));
	    if (particles == NULL) {
                log_severe(
                    FILE_LINE
                    ":  could not realloc particle pointer array (%d length requested)",
                    particleSpaceAllocated * sizeof(void*)
                );
                THROW(std::bad_alloc,);
            }
	}

	/* now store the particle */
	particles[nParticles++] = input;

	/* get ready to go on to the next particle */
	input = iPtr;
    }

    /* this might be too worrysome:
     * free extra space if it is excessive.
     */
    if ( (particleSpaceAllocated - nParticles) >
	 2*mytype::particleSpaceIncrement ) {
	particleSpaceAllocated = nParticles + mytype::particleSpaceIncrement;
	particles = (_Particle **)realloc(particles, particleSpaceAllocated * sizeof(void*));
	if (particles == NULL) {
            log_severe(
                FILE_LINE
                ":  could not realloc particle pointer array (%d length requested)",
                particleSpaceAllocated * sizeof(void*)
            );
            THROW(std::bad_alloc,);
        }
    }

    return total_processed;
}

OctreeTemplate
std::ostream & OctreeContainer<OctreeTemplateArgs>::plotParticlesAndTree(std::ostream & output,
                              const std::string & title,
                              const std::string & tree_plotopt,
                              const std::string & part_plotopt) const {
    std::string plotcmd_buf;
    if (!octree) return output;

    output << "set title '" << title << "'\n"
           << "splot "
           << OctreeClass::createGnuplotLeaveSPLOT(plotcmd_buf, tree_plotopt, octree->leaves)
           << ", '-' using 1:2:3 " << part_plotopt << '\n';
    octree->gnuplotLeavesVol(output << "#Volume definitions:\n");

    return printParticles(output) << "e\n";
}

} /* namespace octree. */
