// -*- c++ -*-
// $Id: OctreeMonitor.h,v 1.2 2005/06/07 10:00:13 olsonse Exp $
/*
 * Copyright 2004 Spencer Olson
 *
 * $Log: OctreeMonitor.h,v $
 * Revision 1.2  2005/06/07 10:00:13  olsonse
 * Many changes over the course of time
 *
 * Revision 1.1.1.1  2005/01/08 04:57:25  olsonse
 * Initial CVS import (finally).
 *
 */

#ifndef OCTREEMONITOR_H
#define OCTREEMONITOR_H

#include "Octree.h"

/** \file
 * This is the first documentation for this file as a whole.
 * */

namespace octree {

/** Octree monitor class.
*/
template <class _DataParticle, class _Particle>
class OctreeMonitor {
  public:
    /** Default contructor. */
    OctreeMonitor();

    inline double distance(const _Particle & p) {
        double center[3];
        this->copyRcm(center);
        double dx = (p.pos[X] - center[X]),
               dy = (p.pos[Y] - center[Y]),
               dz = (p.pos[Z] - center[Z]);
        return sqrt( SQR(dx) + SQR(dy) + SQR(dz) );
    }

    /** Walk the octree and monitor conditions.
     * */
    template <class OTreeType>
    inline void monitor(const OTreeType & octree, const _Particle * p[]) {
        data.startMeasure(*this, p[0]->particleType());
        monitor_recursive(octree, p);
        data.finishMeasure(*this);
    }

    inline void startMeasure(const int & ptype) {
        data.startMeasure(*this, ptype); 
    }

    inline void addMeasure( const _Particle & p ) {
        data.addMeasure(p);
    }

    inline void finishMeasure() {
        data.finishMeasure(*this);
    }

  private:
    template <class OTreeType>
    void monitor_recursive(const OTreeType & octree, const _Particle * p[]);

  public:
    template <class OTreeType>
    inline bool testOverlap(const OTreeType & octree) {
        return vol.overlaps(octree.getVolB());
    }

    template <class T>
    inline bool boundedBy(const T & t) {
        return vol.contains(t.pos);
    }

    inline void copyRcm(double pos[]) const {
        pos[X] = 0.5*(vol.max[X] + vol.min[X]);
        pos[Y] = 0.5*(vol.max[Y] + vol.min[Y]);
        pos[Z] = 0.5*(vol.max[Z] + vol.min[Z]);
    }

    inline double getVolume() const {
        return  (vol.max[X] - vol.min[X])
              * (vol.max[Y] - vol.min[Y])
              * (vol.max[Z] - vol.min[Z]);
    }

    vol_bounds vol;
    _DataParticle data;
};

} /* namespace octree. */

#include "OctreeMonitor.C"

#endif // OCTREEMONITOR_H

