// -*- c++ -*-
// $Id: Particle.C,v 1.3 2005/06/07 10:00:13 olsonse Exp $
/*
 * Copyright 2004 Spencer Olson
 *
 * $Log: Particle.C,v $
 * Revision 1.3  2005/06/07 10:00:13  olsonse
 * Many changes over the course of time
 *
 * Revision 1.2  2005/01/19 11:44:36  olsonse
 * Added Argon data.
 * Added lines to compile on SunOS.
 *
 * Revision 1.1.1.1  2005/01/08 04:57:25  olsonse
 * Initial CVS import (finally).
 *
 */

#include "Particle.h"
#include <olson-tools/physical.h>
#include <olson-tools/readData.h>
#include <math.h>
#include <sstream>

#if (defined(PGCC) || __sun__ == 1)
inline double tgamma(double x) {
    int sign = 0;
    register double lg = lgamma_r(x,&sign);
    return sign * exp(lg);
}
#endif

using namespace physical::constants;

particle_type_info_t particle_info[] = {
    {
	name:		"87Rb",
	size:		1.0,
	mass:		87*amu,
        /*
	diameter:	100*a_0, //
	cross_section:	8*pi*SQR(100*a_0),
        */
        /* REFERENCE FOR S-WAVE cross section and reference temperature:
         * Newbury, Myatt, Wieman Phys. Rev. A 51, R2680 (1995).
         */
	diameter:	4.635 *nm, /* assuming that cross_sect == 8pi d^2 */
	cross_section:	540 /*+- 130*/ *nm*nm,
        T_ref:          25*uK,
        visc_T_law:     (3.0/4.0),      // We'll just guess this value
        gamma_visc_inv: 1.0/tgamma(5.0/2.0 - 3.0/4.0),
        vss_param_inv:  1.0
    },
    {
	name:		"85Rb",
	size:		1.0,
	mass:		85*amu,
	diameter:	100*a_0,
	cross_section:	8*pi*SQR(100*a_0),
        T_ref:          150*uK,         // We'll just guess this value
        visc_T_law:     (3.0/4.0),      // We'll just guess this value
        gamma_visc_inv: 1.0/tgamma(5.0/2.0 - 3.0/4.0),
        vss_param_inv:  1.0
    },
    {
	name:		"Ar",
	size:		1.0,
	mass:		39.948*amu,
	diameter:	4.17e-10*m,
	cross_section:	pi*SQR(4.17e-10*m),
        T_ref:          273*K,
        visc_T_law:     (0.81),
        gamma_visc_inv: 1.0/tgamma(5.0/2.0 - 0.81),
        vss_param_inv:  1.0
        /* REFERENCE for this data:
         * Appendix A from
         * G. A. Bird,
         * \underline{Molecular Gas Dynamics and the Direct Simulation of Gas Flows},
         * 2nd Ed., Clarendon Press, 1994.
         */
    },
    {0, 0, 0, 0, 0}
};

int getParticleTypeIndex(const char * name) {
    int i = 0;
    for ( i = 0;
	  particle_info[i].name && (strcmp(name, particle_info[i].name) != 0);
	  i++
	);

    if (particle_info[i].name) return i;
    else return -1;
}


/*  ** FOR PARTICLEBASE ** */
std::ostream & operator<<(std::ostream & output,const ParticleBase & q){
    output
    	<< q.pos[X] << ' ' << q.pos[Y] << ' ' << q.pos[Z] << ' '
    	<< q.vel[X] << ' ' << q.vel[Y] << ' ' << q.vel[Z] << ' ';
    return output;
} // operator<< ParticleBase


std::ostream & ParticleBase::printVelocity(std::ostream & output){
    output << vel[X] << ' ' << vel[Y] << ' ' << vel[Z];
    return output;
} // operator<< ParticleBase

std::istream & operator>>(std::istream & input, ParticleBase & p) {
    for (unsigned int i = 0; i < 3 ; i++) {
        input >> p.pos[i];
    }
    for (unsigned int i = 0; i < 3 ; i++) {
        input >> p.vel[i];
    }
    return input;
}


/*  ** FOR PARTICLE ** */

/* TODO:  Pick some good value for the mem-limit */
REDEF_NEW_DEL_C(Particle,/* freed mem limit (instances) = */ 150000);

std::istream & operator>>(std::istream & input, Particle *& p) {
    while (read_past_comment(input));

    char line[1024] = {0};
    input.getline(line,sizeof(line));
    std::istringstream ins(line);

    if (!input.good()) return input;

    p = new Particle();
    ins >> ((ParticleBase&) (*p) );
    return input;
}

