// $Id$
/*
 * Copyright 2002-2007 Spencer Olson
 *
 */

#ifndef COLLIDE_H
#define COLLIDE_H

#include <olson-tools/indices.h>
#include <olson-tools/Vector.h>
#include <olson-tools/power.h>
#include <olson-tools/random/random.h>
#include "Particle.h"

class ElasticCollisionModel {
  public:

    template <class _Particle>
    static void collide(_Particle & part1, _Particle & part2) {
    //  collide two particles
        Vector<double,3> VelCM; /* velocity of center of mass. */
        Vector<double,3> VelRelPre; /* relative velocity prior to collision */
        Vector<double,3> VelRelPost;/* relative velocity after collision */
        double SpeedRel = 0;

        /*  first obtain the center of mass velocity components
         *  note that for identical particles, the reduced mass is m/2
         *   RML=SPM(5,LS,MS)/SP(5,MS) == 0.5
         *   RMM=SPM(5,LS,MS)/SP(5,LS) == 0.5
         */

#if 0
        /* after several tests, I can't seem to see any speed improvement of these
         * explicit for loops over the optimized implicit for loops in the
         * Vector<double,3> manipulations. */
        for (int k = X; k <= Z; k++) {
            VelCM[k]     = 0.5 * ( part1.vel[k] + part2.vel[k] );
            VelRelPre[k] = part1.vel[k] - part2.vel[k];
            SpeedRel += SQR(VelRelPre[k]);
        }
        SpeedRel = sqrt(SpeedRel);
#else

        VelCM = V3C(part1.vel) + V3C(part2.vel); VelCM *= 0.5;
        VelRelPre = V3C(part1.vel) - V3C(part2.vel);
        SpeedRel = VelRelPre.abs();
#endif

        /* FIXME:  does the following really have to be compared to 1e-3? */
        if ( fabs( particle_info[part1.type].vss_param_inv - 1.0 ) < 1.e-3 ) {
            // use the VHS logic
            double B = 2.0 * MTRNGrand() - 1.0;
            // B is the cosine of a random elevation angle
            double A = sqrt( 1.0 - SQR(B) );
            VelRelPost[X] = B * SpeedRel;
            double C = 2.0 * M_PI * MTRNGrand();
            // C is a random azimuth angle
            VelRelPost[Y] = A * cos(C) * SpeedRel;
            VelRelPost[Z] = A * sin(C) * SpeedRel;
        } else {
            // use the VSS logic
            double B = 2.0 * fast_pow(MTRNGrand(),particle_info[part1.type].vss_param_inv) - 1.0;
            // B is the cosine of the deflection angle for the VSS model (eqn (11.8)
            double A = sqrt( 1.0 - B*B);
            double C = 2.0 * M_PI * MTRNGrand();
            double COSC = cos(C);
            double SINC = sin(C);
            double D = sqrt( SQR(VelRelPre[Y]) + SQR(VelRelPre[Z]) );
            if ( D > 1.0E-6 ) {
                VelRelPost[X] = B * VelRelPre[X] + A * SINC * D;
                VelRelPost[Y] = B * VelRelPre[Y] + A * (SpeedRel * VelRelPre[Z] * COSC - VelRelPre[X] * VelRelPre[Y] * SINC)/D;
                VelRelPost[Z] = B * VelRelPre[Z] - A * (SpeedRel * VelRelPre[Y] * COSC + VelRelPre[X] * VelRelPre[Z] * SINC)/D;
            } else {
                VelRelPost[X] = B * VelRelPre[X];
                VelRelPost[Y] = A * COSC * VelRelPre[X];
                VelRelPost[Z] = A * SINC * VelRelPre[X];
            }
        // the post-collision rel. velocity components are based on eqn (2.22)
        }

        // VelRelPost is the post-collision relative vel.
#if 0
        for (int k =X; k <= Z; k++) {
            part1.vel[k] = VelCM[k] + ( VelRelPost[k] * 0.5 );
            part2.vel[k] = VelCM[k] - ( VelRelPost[k] * 0.5 );
        }
#else
        //VelRelPost *= 0.5;
        V3C(part1.vel) = VelCM + ( VelRelPost * 0.5 );
        V3C(part2.vel) = VelCM - ( VelRelPost * 0.5 );
#endif

    } // collide
};

#endif /* COLLIDE_H */
