/** \mainpage Gridless DSMC/Octree

        <h3>Technical Reference</h3>
    - \f$\copyright\f$ 1998-2008 Spencer E. Olson

This software is released under the LGPL license.

Note to those who use this software:<br>
I openly acknowledge that the code in this package stands to benefit greatly
from optimization still.  There are portions that could have been implemented
differently, from someone else's point of view, but that someone did not write
it.  I readily accept patches, improvements, and suggestions to further
develop this package.  If interest is high enough, I'll even consider hosting
the revision control on a public server (sourceforge, ...).

I would really appreciate any and all technical feedback, patches,
contributions.  See http://www.umich.edu/~olsonse/ for my best contact info.  


  - \subpage dsmc_intro 
  - \subpage dsmc_pub
  - \subpage dsmc_language 
  - \subpage dsmc_licence "License/Copying Information"

*/

/** \page dsmc_pub Publications


NOTE:  In any work that utilizes this code, please include reference to the
``Gridless DSMC'' paper below (doi:10.1016/j.jcp.2008.04.038) and, when
relevant, also include references to the other papers below.  

Relevant publications for the methods implemented in this software:

- ``Gridless DSMC,''
    S.E. Olson and A.J. Christlieb, J.  Comp. Phys.,
    doi:10.1016/j.jcp.2008.04.038 (2008).   <br>
    URL: http://dx.doi.org/10.1016/j.jcp.2008.04.038<br>
    URL: http://www.umich.edu/~olsonse/dsmc-tree/

- ``Parallel Gridless DSMC,''
    S.E. Olson, A.J. Christlieb, and F.K Fatemi, submitted to Comp. Phys.
    Comm. (2008).   <br>
    URL: http://www.umich.edu/~olsonse/dsmc-tree/

- ``Pressure Driven Evaporative Cooling in Atom Guides,''
    S.E. Olson, G.  Raithel, A.J. Christlieb, and F.K Fatemi, submitted to
    Phys. Rev. (2008).   

*/

/** \page dsmc_intro Introduction
 *
 * This is the introduction.
 *
 * - \subpage dsmc_install
 *
*/

/** \page dsmc_install Installation
 *
 * \section req Requirements
 * - You will need the olson-tools package to compile the DSMC w/ Octree
 *   classes.  This is available at http://www.umich.edu/~olsonse/.  It is
 *   preferable to obtain a version of olson-tools that is closest in SVN
 *   revision number to this package. 
 *
 */

/** \page dsmc_language Programming considerations
 * When I first started developing this code, I spent a considerable time
 * trying to determine which language I would use.  The example code from G.A.
 * Bird that demonstrates grid-based DSMC was implemented in Fortran 77.  Most
 * other atomic physics simulations in my graduate research group were done in
 * Fortran 77.  I had some experience in Pascal
 * (which I quickly repented of), C, C++, Fortran, Java, and some others--some of which
 * aren't worth the bits they are implemented with (in my opinion).  I did
 * not consider interpreted code or virtual machine languages for various
 * reasons.  
 *
 * In the end, I decided to just use all languages, whenever they are most
 * convenient or prove to add significant advantage.  Using C++ as the main
 * language, I started with a significant portion of Fortran and some C code
 * that was linked to from the C++ code.  Over time, most of the code has
 * migrated to C++.  Perhaps when the new D language gets ironed out a bit and
 * good compiler support is available, I will try that.  
 *
 * Programming issues:
 * - \ref fortran "Why not all Fortran?"
 * - \ref cpp "C++ and scientific computing"
 * - \ref templates "C++ Templates"
 *
 * \section cpp C++ 
 * I'll fill this out later, but basically, I do not use RTTI stuff.  
 *
 * \section fortran Why not all Fortran?
 * Short answer:  Because.
 *
 * Long answer:<br>
 * In scientific computing, there is a large bias towards using Fortran.  I,
 * unfortunately, have a slight bias against Fortran:  most Fortran code that
 * I have seen belongs to the most unreadable collection of code out there.  
 * I know that Fortran is
 * case independent, but I cannot understand why most code I have seen is
 * written in all caps, even so many years after the syntactical restrictions
 * in Fortran have eased up a bit.  All caps programs are very hard on my eyes.  I will
 * say that my own Fortran code looks a lot like my C code, despite the case
 * independence.  Freeform is definitely the way to go.
 *
 * I also wanted to make use of object oriented programming, as I find that
 * model very useful.  Furthermore, I find Fortran 95 somewhat unwieldy.  
 *
 *
 * \section templates C++ Template Usage
 * The user of this DSMC package will note the extensive use of C++ templates
 * throughout the entirety of this package.  I am aware that there are some
 * strong feelings among software developers concerning templates.  I began
 * using templates about the same time that I began developing this code.  The
 * main advantage that I see with templates is that a developer can create
 * very generic code that can then be fully optimized at compile time for the
 * specific instance of the code.  
 * 
 * I realize that this line of thought comes with a major assumption that the
 * compiler <em>will</em> actually do the complete optimization correctly.  My
 * experience has shown that different compilers allow this assumption to be
 * valid to different degrees.  The best template optimizing compiler that I
 * have found is from Intel (<code>icpc</code>).  
 *
 * Perhaps my use of C++ templates is a little too pervasive at times, but I 
 * do enjoy the approach anyway.  
 *
 */

/** \page dsmc_licence License/Copying Information
<p>
This software is released under the \ref dsmc_lgpl "LGPL" license. 
</p>

    - \f$\copyright\f$ 1998-2008 Spencer E. Olson

This code is provided under an open source license in order to ensure
the availability of the code and its derivatives.  By using this particular
license, the intent is to foster academic, engineering, and computational
development, while still allowing the code to be used in more real-life
environments.  

NOTE:  In any work that utilizes this code, please include reference to the
``Gridless DSMC'' paper (doi:10.1016/j.jcp.2008.04.038) and, when
relevant, also include references to the other papers listed on the 
\ref dsmc_pub "Publications" page.



<p>
Relevant licenses:
    - \ref dsmc_lgpl "LGPL"
<p>

\section dsmc_lgpl ""
\verbinclude lgpl.txt

*/

